import 'package:bizdoc/core/app.dart';
import 'package:bizdoc/gen_l10n/app_localizations.dart';
import 'package:bizdoc/pages/server_url.dart';
import 'package:bizdoc/services/network.dart';
import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:provider/provider.dart';

import 'base.dart';

class Credentials extends AuthenticationBase {
  final _secureStorage = const FlutterSecureStorage();
  @override
  Widget authorize(BuildContext context) => Consumer<AppState>(
        builder: (context, appState, child) => SignInPage(
          onDone: () => appState.recycle(),
        ),
      );

  @override
  Future<String?> getAccessToken() => _secureStorage.read(key: "token");
}

typedef SignInArgs = void Function();

class SignInPage extends StatefulWidget {
  final SignInArgs onDone;
  const SignInPage({Key? key, required this.onDone}) : super(key: key);
  @override
  _SignInPageState createState() => _SignInPageState();
}

class _SignInPageState extends State<SignInPage> {
  final _secureStorage = const FlutterSecureStorage();
  bool _working = false;
  final _formKey = GlobalKey<FormState>();
  final _usernameCtrl = TextEditingController();
  final _pwdCtrl = TextEditingController();

  @override
  Widget build(BuildContext context) {
    if (_working) {
      return const Center(
        child: CircularProgressIndicator(),
      );
    }
    return Scaffold(
      body: Container(
        margin: const EdgeInsets.symmetric(vertical: 3),
        child: Form(
            key: _formKey,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              crossAxisAlignment: CrossAxisAlignment.stretch,
              children: [
                TextFormField(
                  decoration: InputDecoration(
                    hintText: AppLocalizations.of(context)!.username,
                  ),
                  controller: _usernameCtrl,
                  autofocus: true,
                  validator: (value) {
                    if (value == null || value == '') {
                      return AppLocalizations.of(context)!.required;
                    }
                  },
                ),
                TextFormField(
                  decoration: InputDecoration(
                    hintText: AppLocalizations.of(context)!.password,
                  ),
                  obscureText: true,
                  controller: _pwdCtrl,
                  onSaved: (_) => _ok,
                ),
                ElevatedButton(
                    onPressed: _ok,
                    child: Text(AppLocalizations.of(context)!.oK)),
                Container(
                  alignment: AlignmentDirectional.centerEnd,
                  child: IconButton(
                      onPressed: () =>
                          Navigator.pushNamed(context, ServerUrlPage.routeName),
                      icon: const Icon(Icons.settings)),
                )
              ],
            )),
      ),
    );
  }

  _toggleSigning(bool val) => setState(() {
        _working = val;
      });

  void _ok() async {
    if (_formKey.currentState!.validate()) {
      _toggleSigning(true);
      try {
        await _signIn(_usernameCtrl.text, _pwdCtrl.text, true);

        widget.onDone();
      } catch (e) {
        _toggleSigning(false);
        ScaffoldMessenger.of(context).showSnackBar(
            SnackBar(content: Text(AppLocalizations.of(context)!.err)));
      }
    }
  }

  Future _signIn(String username, String password, bool rememberMe) async {
    final Response response = await network.dio.post('api/formidentity/login',
        data: {
          'username': username,
          'password': password,
          'rememberMe': rememberMe
        });

    var token = response.data['tokenId'];
    if (rememberMe) {
      _secureStorage.write(key: 'username', value: username);
      _secureStorage.write(key: 'rememberMe', value: rememberMe.toString());
      _secureStorage.write(key: 'token', value: token);
    } else {
      _secureStorage.deleteAll();
    }
  }
}
