import 'package:bizdoc/core/extensions.dart';
import 'package:bizdoc/core/hub.dart';
import 'package:bizdoc/models/contracts.dart';
import 'package:bizdoc/models/notifications.dart';
import 'package:bizdoc/models/profile.dart';
import 'package:bizdoc/services/accounts.dart';
import 'package:bizdoc/services/session.dart';
import 'package:flutter/material.dart';
import 'package:timeago/timeago.dart' as time_ago;

/// lists notifications. Listens to [hub] events.
class NotificationPage extends StatefulWidget {
  static const routeName = 'notifications';

  const NotificationPage({Key? key}) : super(key: key);
  @override
  _NotificationPageState createState() => _NotificationPageState();
}

class _NotificationPageState extends State<NotificationPage> {
  final _items = <NotificationBase>[];
  late Future _initialize;
  @override
  void initState() {
    _initialize = _initializeData();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: StreamBuilder<NotifyBase>(
        stream: hub.events.where((event) => event is NotifyBase).cast(),
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            if (snapshot.data is NewNotify) {
              _items.insert(0, snapshot.data!.model);
            } else if (snapshot.data is NotifyUpdate) {
              final item = _items.firstWhereOrNull(
                  (element) => element.id == snapshot.data!.model.id);
              if (item != null) item.update(snapshot.data!.model);
            }
          }
          return FutureBuilder(
            future: _initialize,
            builder: (context, snapshot) {
              if (snapshot.hasData) {
                return ListView.builder(
                    itemCount: _items.length,
                    itemBuilder: (context, index) {
                      final item = _items[index];
                      return ListTile(
                        key: Key(item.time.toString()),
                        title: _construct(item),
                        trailing: Text(time_ago.format(item.time)),
                      );
                    });
              }
              return const LinearProgressIndicator();
            },
          );
        },
      ),
    );
  }

  Future _initializeData() async {
    try {
      final items = await session.notifications();
      _items.addAll(items);
    } catch (e) {}
  }
}

/// construct notification widget of notification model
Widget _construct(NotificationBase item) {
  if (item is CubeAnomaly) {
    return CubeAnomalyWidget(item);
  } else if (item is TextMessage) {
    return TextMessageWidget(item);
  } else if (item is Commented) {
    return CommentedWidget(item);
  } else if (item is Liked) {
    return LikedWidget(item);
  } else if (item is Tagged) {
    return TaggedWidget(item);
  } else if (item is UpcomingEvent) {
    return UpcomingEventWidget(item);
  } else if (item is Nudge) {
    return NudgeWidget(item);
  } else if (item is Escalated) {
    return EscalatedWidget(item);
  } else {
    return StateChangedWidget(item as StateChanged);
  }
}

abstract class NotificationWidgetBase<T extends NotificationBase>
    extends StatelessWidget {
  final T model;

  NotificationWidgetBase(this.model) : super(key: Key(model.time.toString()));
}

class CubeAnomalyWidget extends NotificationWidgetBase<CubeAnomaly> {
  CubeAnomalyWidget(CubeAnomaly model, {Key? key}) : super(model);

  @override
  Widget build(BuildContext context) {
    final cube = session.profile.cubes
        .firstWhere((element) => element.name == model.cube);
    return Text(cube.title);
  }
}

class TaggedWidget extends NotificationWidgetBase<Tagged> {
  TaggedWidget(Tagged model, {Key? key}) : super(model);
  @override
  Widget build(BuildContext context) {
    return FutureBuilder<UserInfo>(
      future: users.find(model.who),
      builder: (context, snapshot) {
        if (snapshot.hasData) return Text(snapshot.data!.name);
        return Container();
      },
    );
  }
}

class LikedWidget extends NotificationWidgetBase<Liked> {
  LikedWidget(Liked model, {Key? key}) : super(model);
  @override
  Widget build(BuildContext context) {
    return FutureBuilder<UserInfo>(
      future: users.find(model.userIds.first),
      builder: (context, snapshot) {
        if (snapshot.hasData) return Text(snapshot.data!.name);
        return Container();
      },
    );
  }
}

class CommentedWidget extends NotificationWidgetBase<Commented> {
  CommentedWidget(Commented model, {Key? key}) : super(model);
  @override
  Widget build(BuildContext context) {
    return FutureBuilder<UserInfo>(
      future: users.find(model.userIds.first),
      builder: (context, snapshot) {
        if (snapshot.hasData) return Text(snapshot.data!.name);
        return Container();
      },
    );
  }
}

class UpcomingEventWidget extends NotificationWidgetBase<UpcomingEvent> {
  UpcomingEventWidget(UpcomingEvent model, {Key? key}) : super(model);
  @override
  Widget build(BuildContext context) {
    return Text(model.number);
  }
}

class NudgeWidget extends NotificationWidgetBase<Nudge> {
  NudgeWidget(Nudge model, {Key? key}) : super(model);
  @override
  Widget build(BuildContext context) {
    return Text(model.number);
  }
}

class StateChangedWidget extends NotificationWidgetBase<StateChanged> {
  StateChangedWidget(StateChanged model, {Key? key}) : super(model);
  @override
  Widget build(BuildContext context) {
    final state = session.profile.states
        .firstWhere((element) => element.name == model.state);
    return Text(state.title);
  }
}

class EscalatedWidget extends NotificationWidgetBase<Escalated> {
  EscalatedWidget(Escalated model, {Key? key}) : super(model);
  @override
  Widget build(BuildContext context) {
    return FutureBuilder<UserInfo>(
      future: users.find(model.substitutingId),
      builder: (context, snapshot) {
        if (snapshot.hasData) return Text(snapshot.data!.name);
        return Container();
      },
    );
  }
}

class TextMessageWidget extends NotificationWidgetBase<TextMessage> {
  TextMessageWidget(TextMessage model, {Key? key}) : super(model);
  @override
  Widget build(BuildContext context) {
    return Text(model.text);
  }
}
