import 'dart:ui';

import 'package:dio/dio.dart';
import 'dart:math';

import '../models/location.dart';
import 'base.dart';

const String googleMapsBaseUri = "https://maps.googleapis.com/maps/api";

class GeoService extends Service {
  /// Find addresses by partial [name]
  Future<Iterable<Address>> addresses(String name,
      {String? countryCode}) async {
    final queryParameters = {'input': name};
    if (countryCode != null) {
      queryParameters['components'] = 'country:$countryCode';
    }
    final data = await _requestAPI('place/autocomplete/json',
        queryParameters: queryParameters);
    return (data['predictions'] as List).map((e) => Address.fromJson(e));
  }

  /// Locate businesses around [latitude] and [longitude] coords
  Future<Iterable<BusinessInfo>> nearby(
      {String? name,
      required double latitude,
      required double longitude,
      int radius = 500}) async {
    final queryParameters = {
      'location': '$latitude,$longitude',
      'radius': radius,
      'language': window.locale
    };
    if (name != null) queryParameters['name'] = name;
    final data = await _requestAPI('place/nearbysearch/json',
        queryParameters: queryParameters);
    return (data['results'] as List).map((e) => BusinessInfo.fromJson(e));
  }

  /// Translate [placeId] into [LatLng] coords
  Future<LatLng> geocode(String placeId) async {
    final data = await _requestAPI('geocode/json',
        queryParameters: {'place_id': placeId});
    final results = data['results'] as Iterable;
    final location = results.elementAt(0)['geometry']['location'];
    return LatLng.fromJson(location);
  }

  ///
  Future<Map> _requestAPI(String uri,
      {required Map<String, dynamic> queryParameters}) async {
    queryParameters['key'] = appState.googleMaps!.apiKey;
    final response = await Dio()
        .get('$googleMapsBaseUri/$uri', queryParameters: queryParameters);
    var data = response.data;
    if (data['status'] != 'OK') throw data['status'];
    return data;
  }

  ///
  Future<Address> address(
      {required double latitude, required double longitude}) async {
    final data = await _requestAPI('geocode/json',
        queryParameters: {'latlng': '$latitude,$longitude'});
    final results = data['results'] as Iterable;
    final location = results.elementAt(0);
    return Address.fromJson(location);
  }
}

/// Singleton access to geo location services
final geo = GeoService();

const earthRadius = 6371;

_degreesToRadians(degrees) => degrees * (pi / 180);
_radiansToDegrees(radians) => radians * (180 / pi);
_greatCircleDistance(angle) => 2 * pi * earthRadius * (angle / 360);

/// Measure distance between two [LatLng] coords
distance(LatLng locationX, LatLng locationY) {
  final locationXLatRadians = _degreesToRadians(locationX.latitude);
  final locationYLatRadians = _degreesToRadians(locationY.latitude);
  final centralSubtendedAngle = _radiansToDegrees(acos(
      sin(locationXLatRadians) * sin(locationYLatRadians) +
          cos(locationXLatRadians) *
              cos(locationYLatRadians) *
              cos(_degreesToRadians(locationX.longitude > locationY.longitude
                  ? (locationX.longitude - locationY.longitude)
                  : locationY.longitude - locationX.longitude))));
  return _greatCircleDistance(centralSubtendedAngle);
}
