import 'dart:io';

import 'package:dio/dio.dart';
import 'package:http_parser/http_parser.dart';
import 'package:intl/intl.dart';
import 'package:mime/mime.dart';
import 'package:path_provider/path_provider.dart';

import '../models/data.dart';
import 'base.dart';

///
class Mailbox extends Service {
  Future flag(int id, bool val) => dio.put('/mailbox/flag/$id?flag=$val');

  Future<RecipientModel> create(String formId,
      {double? longitude, double? latitude}) async {
    final Map<String, dynamic> queryParameters = {'form': formId};
    if (longitude != null) queryParameters['lng'] = longitude;
    if (latitude != null) queryParameters['lat'] = latitude;
    final response = await dio.post('/mail', queryParameters: queryParameters);
    return RecipientModel.fromJson(response.data);
  }

  Future<Iterable<HeaderModel>> messages(
      {String? folderId,
      String? contains,
      QueryType? queryType,
      int? skip,
      int take = 20}) async {
    final Map<String, dynamic> queryParameters = {
      'folderId': folderId,
      'contains': contains,
      'take': take,
      'skip': skip
    };
    if (queryType != null) {
      queryParameters['type'] = queryType.toString().split('.').last;
    }
    final response = await dio.get('/mail', queryParameters: queryParameters);
    final data = List<HeaderModel>.from(
        response.data.map((e) => HeaderModel.fromJson(e)));
    return data;
  }

  Future<RecipientModel> submit(int id, int version, [Map? data]) async {
    try {
      final response = await dio.put('/mail/$id',
          queryParameters: {'version': version, 'go': true},
          data: data,
          options: Options(receiveDataWhenStatusError: true));
      return RecipientModel.fromJson(response.data);
    } on DioError catch (e) {
      handleError(e);
      rethrow;
    }
  }

  Future<RecipientModel> get(int id) async {
    final response = await dio.get('/mail/$id');
    return RecipientModel.fromJson(response.data);
  }

  Future<String> download(int documentId, int fileId,
      {Function(int, int)? progress}) async {
    final tempDir = await getTemporaryDirectory();
    final fullPath = tempDir.path + "/" + fileId.toString();
    await dio.download('/attachments/$documentId/$fileId', fullPath,
        onReceiveProgress: progress, queryParameters: {});
    return fullPath;
  }

  Future<Comment> comment(int documentId,
      {String? text, Map<String, dynamic>? resource}) async {
    final response = await dio.post('/comments/$documentId',
        data: {'resource': resource, 'text': text});
    return Comment.fromJson(response.data);
  }

  Future<Comment> editComment(int documentId, int commentId,
      {String? text, Map<String, dynamic>? resource}) async {
    final response = await dio.put('/comments/$documentId/$commentId',
        data: {'resource': resource, 'text': text});
    return Comment.fromJson(response.data);
  }

  Future deleteComment(int documentId, int commentId) {
    return dio.delete('/comments/$documentId/$commentId');
  }

  Future commentsTyping(int documentId) =>
      dio.put('/comments/typing/$documentId');

  Future<DateTime> seenComments(int documentId) async {
    final response = await dio.put('/comments/viewed/$documentId');
    return DateTime.parse(response.data['time']);
  }

  ///
  Future deleteAttachment(int id, int fileId, int version) {
    return dio.delete('/attachments/$id/$fileId',
        queryParameters: {'version': version});
  }

  Future<Attachment> checkIn(int id, int fileId, int version, File file) async {
    final data = FormData();
    var fileName = file.path.split('/').last;
    fileName = DateFormat('yyMMddhhmmss').format(DateTime.now()) +
        fileName.substring(fileName.lastIndexOf('.'));
    data.files.add(MapEntry(
        fileName,
        await MultipartFile.fromFile(
          file.path,
          contentType: MediaType.parse(lookupMimeType(file.path) ?? ""),
        )));
    try {
      final response = await dio.put('/attachments/$id/$fileId/checkin',
          queryParameters: {'version': version}, data: data);
      return Attachment.fromJson(response.data);
    } on DioError catch (e) {
      handleError(e);
      rethrow;
    }
  }

  Future checkOut(int id, int fileId, int version) {
    return dio.put('/attachments/$id/$fileId/checkout',
        queryParameters: {'version': version});
  }

  Future<List<Attachment>> upload(
      int id, int version, Iterable<File> files) async {
    final data = FormData();
    for (var file in files) {
      var fileName = file.path.split('/').last;
      fileName = DateFormat('yyMMddhhmmss').format(DateTime.now()) +
          fileName.substring(fileName.lastIndexOf('.'));
      data.files.add(MapEntry(
          fileName,
          await MultipartFile.fromFile(
            file.path,
            contentType: MediaType.parse(lookupMimeType(file.path) ?? ""),
          )));
    }
    try {
      final response = await dio.post('/attachments/$id',
          queryParameters: {'version': version}, data: data);
      return List.from(response.data.map((a) => Attachment.fromJson(a)));
    } on DioError catch (e) {
      handleError(e);
      rethrow;
    }
  }

  Future<RecipientModel> save(int id, int version, [Map? data]) async {
    try {
      final response = await dio.put('/mail/$id',
          queryParameters: {'version': version},
          data: data,
          options: Options(receiveDataWhenStatusError: true));
      return RecipientModel.fromJson(response.data);
    } on DioError catch (e) {
      handleError(e);
      rethrow;
    }
  }

  Future<RecipientModel> send(
      int id, int version, String actionId, Map<String, dynamic>? args,
      [Map? data]) async {
    try {
      var queryParameters = <String, dynamic>{
        'version': version,
        'action': actionId
      };
      if (args != null) queryParameters.addAll(args);
      final response = await dio.put('/mail/$id',
          data: data,
          queryParameters: queryParameters,
          options: Options(receiveDataWhenStatusError: true));
      return RecipientModel.fromJson(response.data);
    } on DioError catch (e) {
      handleError(e);
      rethrow;
    }
  }

  discard(int id) {
    try {
      return dio.delete('/mail/$id');
    } on DioError catch (e) {
      handleError(e);
      rethrow;
    }
  }
}

var mailbox = Mailbox();
enum QueryType { unread, recent, flagged }
